var path = require('path')
  , tea = require('tea')
  , httpProxy = require('http-proxy');

var Site = require('./site')
  , FileStore = require('seed-filestore');

var storePath = path.join(__dirname, '..', 'data')
  , AlchemistStorage = new FileStore(storePath);

var Alchemist = Seed.Graph.extend({
  store: AlchemistStorage,
  
  initialize: function(options) {
    var self = this;
    
    options = options || {};
    
    this.define(Site);
    this.activeSites = new Seed.Hash();
    
    var handleProxy = function (req, res, proxy) {
      self._handleProxy.call(self, req, res, proxy);
    };
    
    this.proxy = httpProxy.createServer(handleProxy);
    this.proxyActive = false;
    
    this.flag('autostart', options.autostart || false);
    this.on('add:site:*', this._startSite);
  },
  
  listen: function (port) {
    port = port || 1337;
    this.proxy.listen(port);
    this.proxyActive = true;
    this.emit('log', {
      message: 'Alchemist active on port [' + port +']'
    });
    this._startSites();
  },
  
  _startSite: function (site) {
    if (this.proxyActive && this.flag('autostart')) {
      site.listen();
      this.activeSites.set(site.get('domain'), site.get('port'));
      this.emit('log', {
        message: site.get('name') + '[' + site.get('domain') + '] listening on port [' + site.get('port') + ']'
      });
    }
  },
  
  _startSites: function (callback) {
    var self = this
      , sites = this.all('site')
      , count = 0;
    
    if (!this.proxyActive)
      this.listen(port);
    
    sites.each(function (site) {
      count++;
      self._startSite(site);
      if (count == sites.count && tea.isFunction(callback))
        callback();
    });
  },
  
  _stopSite: function (site) {
    this.activeSites.del(site.get('domain'), site.get('port'));
    this.emit('log', {
        message: site.get('name') + '[' + site.get('domain') + '] has been stopped.'
      });
  },
  
  _stopSites: function (callback) {
    var self = this
      , sites = this.all('site')
      , count = 0;
    
    sites.each(function (site) {
      count++;
      self.stopSite(site);
      if (count == sites.count && tea.isFunction(callback))
        callback();
    });
  },
  
  _handleProxy: function (req, res, proxy) {
    var domain = (req.headers.host || '').replace(/:\d+$/, '')
      , port = this.activeSites.get(domain);
    
    if (port) {
      proxy.proxyRequest(req, res, {
        host: 'localhost',
        port: port
      });
      
      this.get('/site/' + domain)
        .emit('serve', {
          url: req.url,
          time: newDate().getTime()
        });
      
    } else {
      res.statusCode = 404;
      res.write('404 not found');
      res.end();
    }
  }
});

module.exports = Alchemist;